﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;
using System.Collections;
using System.Threading;

namespace NetControl
{
    public partial class Form1 : Form
    {
        Thread t;
        Socket sock;
        const string MULTICAST_IP = "224.0.0.1";
        const UInt16 MULTICAST_PORT = 44444;
        byte portC;
        NetInfoHeader netInf;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                t = new Thread(new ThreadStart(receivePacket));
                t.Start();
            }
            finally
            {
            }
        }

        // wird als extra thread gestartet; empfängt die multicasts
        private void receivePacket()
        {
            sock = new Socket(AddressFamily.InterNetwork,
                        SocketType.Dgram, ProtocolType.Udp);
            IPEndPoint iep = new IPEndPoint(IPAddress.Any, MULTICAST_PORT);
            EndPoint ep = (EndPoint)iep;
            sock.Bind(iep);
            sock.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.AddMembership,
                new MulticastOption(IPAddress.Parse(MULTICAST_IP)));
            byte[] data = new byte[20];
            int recv;
            netInf = new NetInfoHeader();
            while (true)
            {
                recv = sock.ReceiveFrom(data, ref ep);
                netInf.parseHeader(data);
                SetText(labelProtocol, netInf.headerVersion.ToString());
                SetText(labelID, netInf.controlerID.ToString());
                SetText(labelIP, ep.ToString().Split(':')[0]);
                SetValue(progBarADC1, netInf.adc1);
                SetText(labelADC1U, String.Format("{0:0.000}V", netInf.adc1/204.8));
                SetValue(progBarADC2, netInf.adc2);
                SetText(labelADC2U, String.Format("{0:0.000}V", netInf.adc2 / 204.8));
                SetValue(progBarADC3, netInf.adc3);
                SetText(labelADC3U, String.Format("{0:0.000}V", netInf.adc3 / 204.8));
                SetValue(progBarADC4, netInf.adc4);
                SetText(labelADC4U, String.Format("{0:0.000}V", netInf.adc4 / 204.8));
                SetText(labelUpTime, netInf.getUpTimeAsFormatedString);
                SetText(button1, "c0 ist " + (((netInf.portC & 1) == 1) ? "an" : "aus"));
                SetText(button2, "c1 ist " + (((netInf.portC & 2) == 2) ? "an" : "aus"));
                SetText(button3, "c2 ist " + (((netInf.portC & 4) == 4) ? "an" : "aus"));
                SetText(button4, "c3 ist " + (((netInf.portC & 8) == 8) ? "an" : "aus"));
                SetText(button5, "c4 ist " + (((netInf.portC & 16) == 16) ? "an" : "aus"));
                SetText(button6, "c5 ist " + (((netInf.portC & 32) == 32) ? "an" : "aus"));
                SetText(button7, "c6 ist " + (((netInf.portC & 64) == 64) ? "an" : "aus"));
                SetText(button8, "c7 ist " + (((netInf.portC & 128) == 128) ? "an" : "aus"));
                portC = netInf.portC;
            }         
        }

        void sendPacket(byte _digi)
        {
            foreach (IPAddress ipAdd in GetNICIP())
            {
                Socket server = new Socket(AddressFamily.InterNetwork,
                        SocketType.Dgram, ProtocolType.Udp);
                IPEndPoint iep = new IPEndPoint(IPAddress.Parse("224.0.0.1"), 345);
                server.Bind(new IPEndPoint(ipAdd, 32345));
                byte[] data = new byte[1];
                data[0] = _digi;
                server.SendTo(data, iep);
                server.Close();
            }
        }

        // werden benötigt um auf GUI zugreifen zu können
        delegate void SetValueCallback(ProgressBar progBar, int value);
        private void SetValue(ProgressBar progBar, int value)
        {
            if (progBar.InvokeRequired)
            {
                SetValueCallback d = new SetValueCallback(SetValue);
                this.Invoke(d, new object[] { progBar,value });
            }
            else
            {
                progBar.Value = value;
            }
        }

        // werden benötigt um auf GUI zugreifen zu können
        delegate void SetTextCallback(Control _ctl, string _text);
        private void SetText(Control _ctl, string _text)
        {
            if (_ctl.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                this.Invoke(d, new object[] { _ctl, _text });
            }
            else
            {
                _ctl.Text = _text;
            }
        }

        private List<IPAddress> GetNICIP()
        {
            List<IPAddress> result = new List<IPAddress>();
            String strHostName = Dns.GetHostName();
            IPHostEntry iphostentry = Dns.GetHostEntry(strHostName);
            foreach (NetworkInterface f in NetworkInterface.GetAllNetworkInterfaces())
            {
                if (f.OperationalStatus == OperationalStatus.Up)
                {
                    IPInterfaceProperties ipInterface = f.GetIPProperties();
                    foreach (UnicastIPAddressInformation unicastAddress in ipInterface.UnicastAddresses)
                    {
                        if (unicastAddress.IPv4Mask == null) continue;
                        result.Add(unicastAddress.Address);
                    }
                }
            }
            return result;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (t != null) t.Abort();
                sock.Close();
            }
            catch (InvalidCastException ice)
            {
                throw (ice);
            }
        }

        private void buttonAll(object sender, EventArgs e)
        {
            byte bitPos = (byte)(1 << (int.Parse(((Button)(sender)).Name.Substring(((Button)(sender)).Name.Length - 1))-1));
            if ((portC & bitPos) == bitPos)
            {
                sendPacket((byte)(~bitPos & portC));
            }
            else
            {
                sendPacket((byte)(bitPos | portC));
            }
        }
    }

    public class NetInfoHeader
    {
        /*  siehe netinfo.h
         * struct NETINFO_HEADER {
	        unsigned char VERSION;			//	8 Bit	Header Version
	        unsigned char ID;				// 	8 Bit	controler ID
	        unsigned ADC1: 10;   			// 10 Bit 	ADC1 value
	        unsigned ADC2: 10;   			// 10 Bit	ADC2 value
	        unsigned ADC3: 10; 				// 10 Bit	ADC3 value
	        unsigned ADC4: 10; 				// 10 Bit	ADC4 value
	        unsigned char PORT_C;			//  8 Bit	PORTC
	        unsigned long UP_TIME;			// 32 Bit	
	        unsigned char UNDEFINED[6];		// 48 Bit	undefined
									        //-----------------------------------
									        //144 Bits	-> 18 bytes for HEADER
        };

        #define HEADER_SIZE					18  // sizeOf NETCONTROL_HEADER
        */

        public int headerVersion;
        public int controlerID;
        public int adc1 = 0;
        public int adc2;
        public int adc3;
        public int adc4;
        public byte portC;
        public uint upTime;
        public string getUpTimeAsFormatedString
        {
            get { return String.Format("{0:00}:{1:00}:{2:00}", (upTime / 3600) % 24, (upTime / 60) % 60, upTime % 60); }
        }


        private const int headerSize = 18;
        private const int adcOffset = 2;

        public NetInfoHeader()
        {
        }

        public void parseHeader(byte[] data)
        {
            headerVersion = data[0];
            controlerID = data[1];
            adc1 = (data[adcOffset] | ((data[adcOffset + 1] & 3) << 8));
            adc2 = (((data[adcOffset + 1] & 252) >> 2) | ((data[adcOffset + 2] & 15) << 6));
            adc3 = (((data[adcOffset + 2] & 240) >> 4) | ((data[adcOffset + 3] & 63) << 4));
            adc4 = (((data[adcOffset + 3] & 192) >> 6) | (data[adcOffset + 4] << 2));
            portC = data[adcOffset + 5];
            upTime = (uint)((data[adcOffset + 9] << 24) | (data[adcOffset + 8] << 16) | (data[adcOffset + 7] << 8) | data[adcOffset + 6]);
        }
    }
}
